
private import  mango.io.Reader,
                mango.io.Writer,
                mango.io.FileConduit,
                mango.io.CompositeReader, 
                mango.io.CompositeWriter;

/*******************************************************************************

        Use CompositeIO to 'bracket' marshalling of class contents. See 
        documentation on CompositeReader & CompositeWriter for details.

*******************************************************************************/

void main()
{
        // define a serializable class (via interfaces)
        class Wumpus : IReadable, IWritable
        {
                private int     a = 11,
                                b = 112,
                                c = 1024;

                void read (IReader reader)
                {
                        reader.get(a).get(b).get(c);
                }

                void write (IWriter writer)
                {
                        writer.put(a).put(b).put(c);
                }
        }


        // define a serializable class (via interfaces)
        class Wombat : IReadable, IWritable
        {
                private Wumpus  wumpus;
                private char[]  x = "xyz";
                private bool    y = true;
                private float   z = 3.14159;

                this (Wumpus wumpus)
                {
                        this.wumpus = wumpus;
                }

                void read (IReader reader)
                {
                        reader.get(x).get(y).get(z);
                        if (wumpus)
                            wumpus.read (reader);
                }

                void write (IWriter writer)
                {
                        writer.put(x).put(y).put(z);
                        if (wumpus)
                            wumpus.write (writer);
                }
        }

        // construct a Wombat
        Wombat wombat = new Wombat (new Wumpus);

        // open a file for IO
        FileConduit fc = new FileConduit ("random.bin", FileStyle.ReadWriteCreate);

        // construct reader & writer upon the file, with binary IO
        IWriter w = new Writer (fc);
        IReader r = new Reader (fc);

        // write both Wombat & Wumpus (and flush them)
        w.put (wombat).flush();

        // rewind to file start
        fc.seek (0);

        // read both back again
        r.get (wombat);

        fc.close();
}

