/*******************************************************************************

        @file ByteSwap.d
        
        Copyright (C) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        @version        Initial version, October 2004      
        @author         Kris


*******************************************************************************/

module mango.base.ByteSwap;

/*******************************************************************************

        Reverse byte order for specific datum sizes. Note that the
        byte-swap approach avoids alignment issues, so is probably
        faster overall than a traditional 'shift' implementation.

*******************************************************************************/

struct ByteSwap
{       
        version (DigitalMars) version (X86) version (D_InlineAsm)
        {
        /***********************************************************************
        
        ***********************************************************************/

        final static void swap16 (void *dst, uint count)
        {
                asm {
                    mov  ESI, dword ptr [dst];
                    mov  ECX, dword ptr [count];
                    test ECX, ECX;
                    jmp  test;
        loop:
                    mov  AL, byte ptr [ESI+0];
                    xchg AL, byte ptr [ESI+1];
                    mov  byte ptr [ESI+0], AL;
                    add  ESI, 2;
                    sub  ECX, 2;
        test:
                    jne  loop;
                    }
        }

        /***********************************************************************
        
        ***********************************************************************/

        final static void swap32 (void *dst, uint count)
        {
                asm {
                    mov  ESI, dword ptr [dst];
                    mov  ECX, dword ptr [count];
                    test ECX, ECX;
                    jmp  test;
        loop:
                    mov  AL, byte ptr [ESI+0];
                    xchg AL, byte ptr [ESI+3];
                    mov  byte ptr [ESI+0], AL;
                    mov  AL, byte ptr [ESI+1];
                    xchg AL, byte ptr [ESI+2];
                    mov  byte ptr [ESI+1], AL;
                    add  ESI, 4;
                    sub  ECX, 4;
        test:
                    jne  loop;
                    }
        }

        /***********************************************************************
        
        ***********************************************************************/

        final static void swap64 (void *dst, uint count)
        {
                asm {
                    mov  ESI, dword ptr [dst];
                    mov  ECX, dword ptr [count];
                    test ECX, ECX;
                    jmp  test;
        loop:
                    mov  AL, byte ptr [ESI+0];
                    xchg AL, byte ptr [ESI+7];
                    mov  byte ptr [ESI+0], AL;
                    mov  AL, byte ptr [ESI+1];
                    xchg AL, byte ptr [ESI+6];
                    mov  byte ptr [ESI+1], AL;
                    mov  AL, byte ptr [ESI+2];
                    xchg AL, byte ptr [ESI+5];
                    mov  byte ptr [ESI+2], AL;
                    mov  AL, byte ptr [ESI+3];
                    xchg AL, byte ptr [ESI+4];
                    mov  byte ptr [ESI+3], AL;
                    add  ESI, 8;
                    sub  ECX, 8;
        test:
                    jne  loop;
                    }
        }

        /***********************************************************************
        
        ***********************************************************************/

        final static void swap80 (void *dst, uint count)
        {
                asm {
                    mov  ESI, dword ptr [dst];
                    mov  ECX, dword ptr [count];
                    test ECX, ECX;
                    jmp  test;
        loop:
                    mov  AL, byte ptr [ESI+0];
                    xchg AL, byte ptr [ESI+9];
                    mov  byte ptr [ESI+0], AL;
                    mov  AL, byte ptr [ESI+1];
                    xchg AL, byte ptr [ESI+8];
                    mov  byte ptr [ESI+1], AL;
                    mov  AL, byte ptr [ESI+2];
                    xchg AL, byte ptr [ESI+7];
                    mov  byte ptr [ESI+2], AL;
                    mov  AL, byte ptr [ESI+3];
                    xchg AL, byte ptr [ESI+6];
                    mov  byte ptr [ESI+3], AL;
                    mov  AL, byte ptr [ESI+4];
                    xchg AL, byte ptr [ESI+5];
                    mov  byte ptr [ESI+4], AL;
                    add  ESI, 10;
                    sub  ECX, 10;
        test:
                    jne  loop;
                    }
        }
        }

     else

        {
        /***********************************************************************
        
        ***********************************************************************/

        final static void swap16 (void *dst, uint count)
        {
                ubyte* p = cast(ubyte*) dst;
                while (count)
                      {
                      byte b = p[0];
                      p[0] = p[1];
                      p[1] = b;

                      p += short.sizeof;
                      count -= short.sizeof;
                      }
        }

        /***********************************************************************
        
        ***********************************************************************/

        final static void swap32 (void *dst, uint count)
        {
                ubyte* p = cast(ubyte*) dst;
                while (count)
                      {
                      ubyte b = p[0];
                      p[0] = p[3];
                      p[3] = b;

                      b = p[1];
                      p[1] = p[2];
                      p[2] = b;

                      p += int.sizeof;
                      count -= int.sizeof;
                      }
        }

        /***********************************************************************
        
        ***********************************************************************/

        final static void swap64 (void *dst, uint count)
        {
                ubyte* p = cast(ubyte*) dst;
                while (count)
                      {
                      ubyte b = p[0];
                      p[0] = p[7];
                      p[7] = b;

                      b = p[1];
                      p[1] = p[6];
                      p[6] = b;
        
                      b = p[2];
                      p[2] = p[5];
                      p[5] = b;
        
                      b = p[3];
                      p[3] = p[4];
                      p[3] = b;

                      p += long.sizeof;
                      count -= long.sizeof;
                      }
        }

        /***********************************************************************
        
        ***********************************************************************/

        final static void swap80 (void *dst, uint count)
        {
                ubyte* p = cast(ubyte*) dst;
                while (count)
                      {
                      ubyte b = p[0];
                      p[0] = p[9];
                      p[9] = b;

                      b = p[1];
                      p[1] = p[8];
                      p[8] = b;

                      b = p[2];
                      p[2] = p[7];
                      p[7] = b;
        
                      b = p[3];
                      p[3] = p[6];
                      p[6] = b;
        
                      b = p[4];
                      p[4] = p[5];
                      p[5] = b;

                      p += real.sizeof;
                      count -= real.sizeof;
                      }
        }

        }
}
