/*******************************************************************************

        @file ArrayAllocator.d
        
        Copyright (C) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        @version        Initial version, October 2004      
        @author         Kris


*******************************************************************************/

module mango.io.ArrayAllocator;

private import  mango.io.Reader;

private import  mango.utils.HeapSlice;

private import  mango.io.model.IBuffer,
                mango.io.model.IReader;

/*******************************************************************************

*******************************************************************************/

class SimpleAllocator : IArrayAllocator
{
        private IReader reader;

        /***********************************************************************
        
        ***********************************************************************/

        this (IReader reader)
        {
                this.reader = reader;
        }

        /***********************************************************************
        
        ***********************************************************************/

        void reset ()
        {
        }

        /***********************************************************************
        
        ***********************************************************************/

        void allocate (void[]* x, BufferConverter decoder)
        {       
                uint len = reader.length;
                *x = new void [len];
                //printf ("allocated %d bytes\n", len);
                decoder (*x, len);
        }
}


/*******************************************************************************

*******************************************************************************/

class BufferAllocator : IArrayAllocator
{
        private BufferConverter raw;
        private uint            width;
        private IReader         reader;

        /***********************************************************************
        
        ***********************************************************************/

        this (IReader reader, int width = 0)
        {
                this.width = width;
                this.reader = reader;
                raw = &(cast(Reader) reader).read;
        }

        /***********************************************************************
        
        ***********************************************************************/

        void reset ()
        {
                IBuffer buffer = reader.getBuffer;

                // ensure there's enough room for another record
                if (buffer.writable < width)
                    buffer.compress ();
        }

        /***********************************************************************
        
        ***********************************************************************/

        void allocate (void[]* x, BufferConverter decoder)
        {       
                uint len = reader.length;

                if (decoder == raw)
                   {
                   *x = reader.getBuffer.get (len);
                   //printf ("sliced %d bytes from buffer\n", len);
                   }
                else
                   {
                   *x = new void [len];
                   //printf ("allocated %d bytes\n", len);
                   decoder (*x, len);
                   }
        }
}


/*******************************************************************************

*******************************************************************************/

class SliceAllocator : HeapSlice, IArrayAllocator
{
        private IReader reader;

        /***********************************************************************
        
        ***********************************************************************/

        this (IReader reader, int width)
        {
                super (width);
                this.reader = reader;
        }

        /***********************************************************************
        
        ***********************************************************************/

        void reset ()
        {
                super.reset ();
        }

        /***********************************************************************
        
        ***********************************************************************/

        void allocate (void[]* x, BufferConverter decoder)
        {       
                uint len = reader.length;

                expand (len);
                *x = slice (len);
                //printf ("sliced %d bytes from heap\n", len);
                decoder (*x, len);
        }
}


