/*******************************************************************************

        @file TextReader.d
        
        Copyright (C) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        @version        Initial version, March 2004      
        @author         Kris


*******************************************************************************/

module mango.io.TextReader;

private import  mango.io.Token,
                mango.io.Tokenizer,
                mango.io.Exception,
                mango.io.AbstractReader;

/*******************************************************************************

        Grok readable input from a stream. All input is tokenized from the
        associated buffer, and converted as necessary into the destination 
        location. A SpaceTokenizer is used by default, but you can choose
        an alternative (such as a comma-delimiting tokenizer).

*******************************************************************************/

class TextReader : AbstractReader
{
        // an individual token
        private Token token;

        // internal tokenizer instance
        private ITokenizer tokenizer;

        /***********************************************************************
        
                Construct a TextReader on the provided buffer, using the
                specified Tokenizer instead of the default one.

        ***********************************************************************/

        this (IBuffer buffer, ITokenizer tokenizer)
        {
                super (buffer);
                this.token = new Token;
                this.tokenizer = tokenizer;
     
                numeric.int1 = &int1;
                numeric.int8 = 
                numeric.int8u = &int8;
                numeric.int16 =
                numeric.int16u = &int16;
                numeric.int32 = 
                numeric.int32u = &int32;
                numeric.int64 = 
                numeric.int64u = &int64;

                numeric.float32 = &float32;
                numeric.float64 = &float64;
                numeric.float80 = &float80;
        }
     
        /***********************************************************************
        
                Construct a TextReader on the given buffer

        ***********************************************************************/

        this (IBuffer buffer)
        {
                this (buffer, Tokenizers.space);
        }
     
        /***********************************************************************
                
                Construct a TextReader upon the buffer associated with the
                given conduit.

        ***********************************************************************/

        this (IConduit conduit)
        {
                this (conduit.createBuffer);
        }

        /***********************************************************************
        
        ***********************************************************************/
        
        override char[] toString()
        {
                return "text reader";
        }

        /***********************************************************************
        
                Internal method to isolate the next token.

        ***********************************************************************/

        private final Token next ()
        {
                if (tokenizer.next (buffer, token) && token.getLength > 0)
                    return token;    
                throw new TokenException ("unexpected end of input");
        }

        /***********************************************************************
        
        ***********************************************************************/

        void int1 (void* src, uint count)
        {     
                while (count)
                      {
                      * cast(bool*) src = (next.toString == "true");
                      ++src;
                      --count;
                      }
        }

        /***********************************************************************
        
        ***********************************************************************/

        void int8 (void* src, uint count)
        {     
                while (count)
                      {
                      * cast(byte*) src = next.toInt();
                      src += byte.sizeof;
                      count -= byte.sizeof;
                      }
        }

        /***********************************************************************
        
        ***********************************************************************/

        void int16 (void* src, uint count)
        {     
                while (count)
                      {
                      * cast(short*) src = next.toInt();
                      src += short.sizeof;
                      count -= short.sizeof;
                      }
        }

        /***********************************************************************
        
        ***********************************************************************/

        void int32 (void* src, uint count)
        {     
                while (count)
                      {
                      * cast(int*) src = next.toInt();
                      src += int.sizeof;
                      count -= int.sizeof;
                      }
        }

        /***********************************************************************
        
        ***********************************************************************/

        void int64 (void* src, uint count)
        {     
                while (count)
                      {
                      * cast(long*) src = next.toLong();
                      src += long.sizeof;
                      count -= long.sizeof;
                      }
        }

        /***********************************************************************
        
        ***********************************************************************/

        void float32 (void* src, uint count)
        {     
                while (count)
                      {
                      * cast(float*) src = next.toReal();
                      src += float.sizeof;
                      count -= float.sizeof;
                      }
        }

        /***********************************************************************
        
        ***********************************************************************/

        void float64 (void* src, uint count)
        {     
                while (count)
                      {
                      * cast(double*) src = next.toReal();
                      src += double.sizeof;
                      count -= double.sizeof;
                      }
        }

        /***********************************************************************
        
        ***********************************************************************/

        void float80 (void* src, uint count)
        {     
                while (count)
                      {
                      * cast(real*) src = next.toReal();
                      src += real.sizeof;
                      count -= real.sizeof;
                      }
        }

        /***********************************************************************
        
        ***********************************************************************/

        override IReader get (inout char[] x) 
        { 
                // dup the input string, to avoid surprises
                x = next.toString (false);
                return this;
        }

        /***********************************************************************
        
                @todo - Tokenizer needs to handle wchar[] before this will
                        operate correctly

        ***********************************************************************/

        override IReader get (inout wchar[] x) 
        { 
                assert (0);
                return this;
        }

        /***********************************************************************
        
                @todo - Tokenizer needs to handle dchar[] before this will
                        operate correctly

        ***********************************************************************/

        override IReader get (inout dchar[] x) 
        { 
                assert (0);
                return this;
        }
}
