/*******************************************************************************

        @file FileAppender.d

        Copyright (C) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      
        @version        Initial version, May 2004
        @author         Kris


*******************************************************************************/

module mango.log.FileAppender;

private import  mango.log.Appender;

private import  mango.io.FileConduit,
                mango.io.DisplayWriter;

/*******************************************************************************

        Append log messages to a file. This basic version has no rollover 
        support, so it just keeps on adding to the file. There's also a
        RollingFileAppender that may suit your needs.

*******************************************************************************/

public class FileAppender : Appender
{
        private static uint     mask;
        private DisplayWriter   writer;
        private IConduit        conduit;

        /***********************************************************************
                
                Get a unique fingerprint for this class

        ***********************************************************************/

        static this()
        {
                mask = nextMask();
        }

        /***********************************************************************

        ***********************************************************************/

        protected this (){}

        /***********************************************************************
                
                Create a basic FileAppender to a file with the specified 
                path.

        ***********************************************************************/

        this (char[] path)
        {
                setConduit (new FileConduit (path, FileStyle.WriteAppending));
        }

        /***********************************************************************
                
                Create a basic FileAppender to a file with the specified 
                path, and with the given Layout

        ***********************************************************************/

        this (char[] path, Layout layout)
        {
                this (path);
                setLayout (layout);
        }

        /***********************************************************************
                
                Make sure the file is closed when we're GC'd

        ***********************************************************************/

        ~this ()
        {
                close();
        }

        /***********************************************************************
                
                Return the conduit

        ***********************************************************************/

        IConduit getConduit ()
        {
                return conduit;
        }

        /***********************************************************************
                
                Set the conduit

        ***********************************************************************/

        IWriter setConduit (IConduit conduit)
        {
                // close any existing conduit
                close ();

                // create a new writer upon this conduit
                this.conduit = conduit;
                return (writer = new DisplayWriter (conduit));
        }

        /***********************************************************************
                
                Return the fingerprint for this class

        ***********************************************************************/

        uint getMask ()
        {
                return mask;
        }

        /***********************************************************************
                
                Return the name of this class

        ***********************************************************************/

        char[] getName ()
        {
                return this.classinfo.name;
        }
                
        /***********************************************************************
                
                Append an event to the output.
                 
        ***********************************************************************/

        synchronized void append (Event event)
        {
                writer.put (getLayout.format(event)).cr().flush();
        }

        /***********************************************************************
                
                Close the file associated with this Appender

        ***********************************************************************/

        synchronized void close ()
        {
                if (conduit)
                   {
                   conduit.close();
                   conduit = null;
                   }
        }
}
